@@ -1,32 +1,49 @@
 /*
  *	machine.h
  *
- *	Definition of Machine Depended Identifier
+ *	definitions of machine depended identifier
  */
 
 #ifndef	__MACHINE_H_
 #define	__MACHINE_H_
 
-#ifdef	MSDOS
 #undef	MSDOS
+#if	(DOSV || PC98 || J3100)
+#define	MSDOS			1
+#else
+#define	MSDOS			0
 #endif
-#define	MSDOS		(DOSV || PC98 || J3100)
 
 #if	MSDOS
+#define	NOMULTIKANJI
+#define	PATHNOCASE
+#define	COMMNOCASE
+#define	ENVNOCASE
+#define	NOUID
+#define	NODIRLOOP
+#define	NOSYMLINK
+#define	BSPATHDELIM
+#define	USECRNL
+#define	CWDINPATH
+#define	DOUBLESLASH
 #define	NOTZFILEH
 #define	USETIMEH
+#define	NOFTPH
 #define	USEUTIME
+#define	NOFLOCK
+#define	NOSYSLOG
 #define	USEMKTIME
+#define	NOSETPGRP
 #define	USESTRERROR
-#define	SENSEPERSEC	20
+#define	SENSEPERSEC		20
 # if	defined (__TURBOC__) && defined (__WIN32__)
 # define	BCC32
 # endif
-# ifdef	__GNUC__
+# if	defined (__GNUC__)
 # define	USERESOURCEH
 # define	SIGFNCINT
 #  ifndef	DJGPP
-#  define	DJGPP	1
+#  define	DJGPP		1
 #  else
 #  define	USEREGCOMP
 #  endif
@@ -37,11 +54,8 @@
 # define	NOFILEMODE
 # define	NOUNISTDH
 # endif	/* !__GNUC__ */
-# if	!defined (__GNUC__) || (DJGPP >= 2)
-typedef unsigned int	u_int;
-typedef unsigned char	u_char;
-typedef unsigned short	u_short;
-typedef unsigned long	u_long;
+# if	!defined (__GNUC__) || (defined (DJGPP) && DJGPP >= 2)
+# define	NOUINT
 # endif
 # ifdef	__TURBOC__
 #  ifndef	BCC32
@@ -60,11 +74,9 @@ typedef long	off_t;
 # if	defined (DJGPP) || defined (BCC32)
 # define	PROTECTED_MODE
 # endif
-#define	_SC_	'\\'
-#define	_SS_	"\\"
-#else	/* !MSDOS */
-#define	_SC_	'/'
-#define	_SS_	"/"
+# if	!defined (DJGPP) || (DJGPP < 2)
+# define	NOSELECT
+# endif
 #endif	/* !MSDOS */
 
 #ifndef	NEAR
@@ -75,6 +87,7 @@ typedef long	off_t;
 #define	CODEEUC
 #define	CPP7BIT
 #define	USELEAPCNT
+#define	SOCKETLIB		"-lsocket -lnsl"
 #include <sys/param.h>
 # if	!defined (MAXHOSTNAMELEN) \
 || defined (USGr4) || defined (__svr4__) || defined (__SVR4)
@@ -92,11 +105,13 @@ typedef long	off_t;
 # define	NODNAMLEN
 # define	DNAMESIZE	1
 # define	NOTMGMTOFF
+# define	USEINSYSTMH
 # define	USESTATVFSH
 # define	USESTATVFS_T
 # define	USEMNTTABH
 # define	USEMKTIME
 # define	SIGFNCINT
+# define	USEINETPTON
 # else	/* MAXHOSTNAMELEN && !USGr4 && !__svr4__ && !__SVR4 */
 # define	BSD43
 # define	OSTYPE		"SUN_OS"
@@ -157,7 +172,7 @@ typedef long	off_t;
 #define	SYSV
 #define	OSTYPE			"IRIX"
 #define	CODEEUC
-# if	defined (_COMPILER_VERSION) && (_COMPILER_VERSION >= 600)
+# if	defined (_COMPILER_VERSION) && ((_COMPILER_VERSION) >= 600)
 # define	EXTENDCCOPT	"-32 -O"
 # endif
 # if	!defined (SYSTYPE_SVR4) && !defined (_SYSTYPE_SVR4)
@@ -168,10 +183,10 @@ typedef long	off_t;
 #define	DECLERRLIST
 #define	USEMKDEVH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	NOTMGMTOFF
 #define	USESTATFSH
-#define	STATFSARGS	4
+#define	STATFSARGS		4
 #define	USERE_COMP
 #define	USEMKTIME
 #define	USESYSCONF
@@ -188,15 +203,39 @@ typedef long	off_t;
 || defined (__H3050) || defined (__H3050R) || defined (__H3050RX)
 #define	SYSV
 #define	OSTYPE			"HPUX"
-#define	EXTENDCCOPT		"-D_FILE_OFFSET_BITS=64"
-#define	USETERMINFO
-#define	TERMCAPLIB		"-lcurses"
-#define	STRICTSTDC
+# if	defined (__CLASSIC_C__)
+# define	NOSTDC
+# endif
+# if	defined (__STDC_EXT__)
+# define	EXTENDCCOPT	"-D_FILE_OFFSET_BITS=64"
+# endif
+/*
+ *	This is a fake '#if' for some buggy HP-UX terminfo library.
+ *	Will you please define 'BUGGY_HPUX' manually on such environment.
+ *
+ *	Some HP-UX has buggy libcurses/terminfo, to use libtermcap/termcap.
+ *	Another HP-UX has buggy libtermlib/termcap, to use libcurses/terminfo.
+ *	Or another HP-UX does not support termcap at all.
+ *	If you use HP-UX and the terminal trouble occured,
+ *	you should try to define 'BUGGY_HPUX' here and to re-compile.
+ *
+#define	BUGGY_HPUX
+ *
+ */
+# if	!defined (__HP_cc)
+# define	BUGGY_HPUX	/* Maybe HP-UX 10.20 */
+# endif
+# if	!defined (BUGGY_HPUX)
+# define	USETERMINFO
+# define	TERMCAPLIB	"-lcurses"
+# endif
+#define	BUGGYMAKE
 #define	USEPID_T
 #define	NOSIGLIST
 #define	NOTZFILEH
-#define	USEMKNODH
+#define	USETIMEH
 #define	NOTMGMTOFF
+#define	USEINSYSTMH
 #define	USEREGCOMP
 #define	NOTERMVAR
 #define	USEUTIME
@@ -210,6 +249,9 @@ typedef long	off_t;
 #define	USESETVBUF
 #define	SIGFNCINT
 #define	USESTRERROR
+#define	USESETSID
+#define	USESETRESUID
+#define	USESETRESGID
 #endif
 
 #if	defined (nec_ews) || defined (_nec_ews)
@@ -259,7 +301,7 @@ typedef long	off_t;
 #define	USEREGCMP
 #define	USERESOURCEH
 #define	WAITKEYPAD		720
-#define	WAITMETA		720
+#define	WAITKANJI		720
 #endif
 
 #if	defined (uniosb)
@@ -297,7 +339,8 @@ typedef long	off_t;
 && !defined (__FreeBSD__) && !defined (__NetBSD__) && !defined (__OpenBSD__)
 #define	CODEEUC
 #define	TARUSESPACE
-#define	EXTENDLIB	"-lc_r"
+#define	NOFUNCMACRO
+#define	EXTENDLIB		"-lc_r"
 #define	USEMNTINFOR
 # if	defined (SYSTYPE_BSD)
 # define	BSD43
@@ -335,6 +378,7 @@ typedef long	off_t;
 #define	USEMNTCTL
 #define	USERE_COMP
 # if	defined (_AIX41)
+# define	HAVELONGLONG
 # define	USESTATVFSH
 # define	USEMKTIME
 # define	SIGFNCINT
@@ -382,10 +426,10 @@ typedef long	off_t;
 #define	CODEEUC
 #define	TERMCAPLIB		"-ltermcap"
 #define	USESTATFSH
-#define	STATFSARGS	4
+#define	STATFSARGS		4
 #define	USESYSDIRH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	USERE_COMP
 #endif
 
@@ -397,7 +441,7 @@ typedef long	off_t;
 #define	REGEXPLIB		"-lgen"
 #define	USEMKDEVH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	NOTMGMTOFF
 #define	USESTATVFSH
 #define	USEMNTTABH
@@ -412,7 +456,7 @@ typedef long	off_t;
 #define	REGEXPLIB		"-lgen"
 #define	USEMKDEVH
 #define	NODNAMLEN
-#define	DNAMESIZE	1
+#define	DNAMESIZE		1
 #define	NOTMGMTOFF
 #define	USESTATVFSH
 #define	USEMNTTABH
@@ -432,17 +476,70 @@ typedef long	off_t;
 #define	USERESOURCEH
 #endif
 
+#if	defined (__CYGWIN__)
+#define	POSIX
+#define	OSTYPE			"CYGWIN"
+#define	DEFKCODE		"utf8"
+#define	UTF8DOC
+#define	EXTENDCCOPT		"-O -D_FILE_OFFSET_BITS=64"
+#define	PATHNOCASE
+#define	COMMNOCASE
+#define	USECRNL
+#define	DOUBLESLASH
+#define	USEMANLANG
+#define	LANGWIDTH		2
+#define	BSDINSTALL
+#define	TARUSESPACE
+/*
+ *	This is a fake '#if' for obsolete version of Cygwin.
+ *	Will you please define 'OldCygwin' manually on Cygwin.
+ *
+ *	Some old Cygwin has no libncurses, to use libtermcap.
+ *	Latest distribution has no libtermcap, to use libncurses.
+ *	If you use older Cygwin without libncurses,
+ *	you should try to define 'OldCygwin' here and to re-compile.
+ *
+#define	OldCygwin
+ *
+ */
+# if	defined (OldCygwin)
+# define	TERMCAPLIB	"-ltermcap"
+# else
+# define	TERMCAPLIB	"-lncurses"
+# endif
+#define	NOSIGLIST
+#define	DECLERRLIST
+#define	HAVECLINE
+#define	SYSVDIRENT
+#define	NODNAMLEN
+#define	NODRECLEN
+#define	HAVETIMEZONE
+#define	NOTMGMTOFF
+#define	USEREGCOMP
+#define	USESETENV
+#define	USEMKTIME
+#define	DEFFDSETSIZE
+#define	USETCGETPGRP		/* for Cygwin buggy ioctl */
+#define	SIGFNCINT
+#define	USESOCKLEN
+#define	USEINETATON
+#define	WAITKEYPAD		720
+#define	WAITTERMINAL		36
+#endif
+
 #if	defined (linux)
 #define	POSIX
 #define	OSTYPE			"LINUX"
+#define	UTF8LANG		"utf8-iconv"
 #define	CODEEUC
 # if	defined (alpha) || defined (__alpha) || defined (__alpha__)
 /* Linux/Alpha need not support LFS and cannot use statfs(2) with LFS */
-# define	EXTENDCCOPT		"-O"
+# define	EXTENDCCOPT	"-O"
 # else
-# define	EXTENDCCOPT		"-O -D_FILE_OFFSET_BITS=64"
+# define	EXTENDCCOPT	"-O -D_FILE_OFFSET_BITS=64"
 # endif
 #define	USEMANLANG
+#define	LANGWIDTH		2
 #define	BSDINSTALL
 #define	TARUSESPACE
 /*
@@ -458,14 +555,15 @@ typedef long	off_t;
  *
  */
 # if	defined (Slackware)
-# define	TERMCAPLIB		"-ltermcap"
+# define	TERMCAPLIB	"-ltermcap"
 # else
-# define	TERMCAPLIB		"-lncurses"
+# define	TERMCAPLIB	"-lncurses"
 # endif
 #define	DECLSIGLIST
 #define	DECLERRLIST
 #define	NOTZFILEH
 #define	USETIMEH
+#define	HAVECLINE
 #define	SYSVDIRENT
 #define	NODNAMLEN
 #define	HAVETIMEZONE
@@ -477,10 +575,13 @@ typedef long	off_t;
 && !defined (ia64) && !defined (__ia64) && !defined (__ia64__) \
 && !defined (x86_64) && !defined (__x86_64) && !defined (__x86_64__) \
 && !defined (s390x) && !defined (__s390x) && !defined (__s390x__) \
-&& !defined (CONFIG_ARCH_S390X)
+&& !defined (CONFIG_ARCH_S390X) \
+&& (!defined (PPC) || !defined (__GNUC__) || (__GNUC__) >= 3)	/* for bug */
 # define	USELLSEEK
 # endif
 #define	SIGFNCINT
+#define	USESOCKLEN
+#define	USEINETATON
 #endif
 
 #if	defined (__FreeBSD__) && defined (__powerpc__)
@@ -496,8 +597,10 @@ typedef long	off_t;
 #define	USEMNTINFO
 #define	USERE_COMP
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__FreeBSD__) && !defined (OSTYPE) && !defined (__BOW__)
@@ -513,15 +616,20 @@ typedef long	off_t;
 #define	NOTZFILEH
 #define	USEMOUNTH
 #define	USEMNTINFO
-# if	__FreeBSD__ < 3
+# if	(__FreeBSD__) >= 5
+# define	LANGWIDTH		2
+# endif
+# if	(__FreeBSD__) < 3
 # define	USEVFCNAME
 # else
 # define	USEFFSTYPE
 # endif
 #define	USERE_COMP
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__NetBSD__)
@@ -534,17 +642,27 @@ typedef long	off_t;
 #define	DECLSIGLIST
 #define	DECLERRLIST
 #define	USELEAPCNT
-#define	USEMOUNTH
-#define	USEMNTINFO
 #define	USEFFSTYPE
 #define	USERE_COMP
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #include <sys/param.h>
 # if	defined (NetBSD1_0) && (NetBSD1_0 < 1)
 # define	USEFFSIZE
 # endif
+# if	defined (__NetBSD_Version__) && (__NetBSD_Version__ >= 104000000)
+# define	USESOCKLEN
+# endif
+# if	defined (__NetBSD_Version__) && (__NetBSD_Version__ >= 300000000)
+# define	USESTATVFSH
+# define	USEGETVFSTAT
+# else
+# define	USEMOUNTH
+# define	USEMNTINFO
+# endif
 #endif
 
 #if	defined (__bsdi__)
@@ -560,9 +678,11 @@ typedef long	off_t;
 #define	USEMNTINFO
 #define	USERE_COMP
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
+#define	USEINETATON
 #include <sys/param.h>
-# if	!defined (BSD) || (BSD < 199306)
+# if	!defined (BSD) || ((BSD) < 199306)
 # define	USEFFSIZE
 # endif
 #endif
@@ -577,8 +697,8 @@ typedef long	off_t;
 #define	DECLERRLIST
 #define	USEMOUNTH
 #define	USEMNTINFO
-# ifdef	__FreeBSD__
-#  if	__FreeBSD__ < 3
+# if	defined (__FreeBSD__)
+#  if	(__FreeBSD__) < 3
 #  define	USEVFCNAME
 #  else
 #  define	USEFFSTYPE
@@ -586,8 +706,10 @@ typedef long	off_t;
 # endif	/* __FreeBSD__ */
 #define	USEREGCOMP
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__OpenBSD__)
@@ -605,14 +727,19 @@ typedef long	off_t;
 #define	USEFFSTYPE
 #define	USERE_COMP
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USEINETATON
 #endif
 
 #if	defined (__APPLE__) && defined (__MACH__) && !defined (OSTYPE)
 #define	BSD44
 #define	OSTYPE			"DARWIN"	/* aka Mac OS X */
+#define	DEFKCODE		"utf8-mac"
+#define	UTF8DOC
 #define	USEMANLANG
+#define	LANGWIDTH		2
 #define	BSDINSTALL
 #define	TARFROMPAX
 #define	TERMCAPLIB		"-lcurses"
@@ -624,8 +751,104 @@ typedef long	off_t;
 #define	USEFFSTYPE
 #define	USEREGCOMP
 #define	USESETENV
+#define	SELECTRWONLY
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	SIGFNCINT
+#define	USESOCKLEN
+#define	USEINETATON
+#endif
+
+#if	defined (__minix)
+#define	OSTYPE			"MINIX"
+#ifndef	_MINIX
+#define	_MINIX			1
+#endif
+#include <minix/config.h>
+# if	defined (_VCS_REVISION)
+# define	MINIX32
+#  ifndef	_XOPEN_SOURCE
+#  define	_XOPEN_SOURCE	520
+#  endif
+# else
+# define	MINIX3
+# endif
+#define	CODEEUC
+#define	EXTENDCCOPT		"-O -D_MINIX=1"
+#define	USEPID_T
+#define	NOSIGLIST
+#define	USESELECTH
+#define	USETERMIOS
+#define	USEDEVPTY
+#define	SYSVDIRENT
+#define	NODNAMLEN
+#define	DNAMESIZE		1
+#define	USESETENV
+#define	USEUTIME
+#define	USEFCNTLOCK
+#define	USEMKTIME
+#define	USESYSCONF
+#define	USEWAITPID
+#define	USESIGACTION
+#define	USESIGPMASK
+#define	USETIMES
+#define	GETPGRPVOID
+#define	NOSETPGRP
+#define	USESETVBUF
+#define	SIGFNCINT
+#define	USESTRERROR
+#define	GETTODNULL
+#define	USESETSID
+#define	USESOCKLEN
+#define	NOSENDFLAGS
+#define	USEGETGROUPS
+# ifdef	MINIX32
+# define	BUGGYSIGNALH
+# define	USELEAPCNT
+# define	USESTDARGH
+# define	USESTATVFSH
+# define	USEGETFSENT
+# define	USEULIMITH
+# define	USEINETPTON
+# else
+# define	CPP7BIT
+# define	TERMCAPLIB	"-lcurses"
+# define	OLDARGINT
+# define	NOFUNCMACRO
+# define	NOLONGLONG
+# define	NOTZFILEH
+# define	USETIMEH
+# define	NOTMGMTOFF
+# define	NOSTBLKSIZE
+# define	NOFTPH
+# define	USESTATFSH
+# define	NOFBLOCKS
+# define	NOFBFREE
+# define	NOFFILES
+# define	USEFSTATFS
+# define	USEREADMTAB
+# define	NOTERMVAR
+# define	NOFTRUNCATE
+# define	NOSYSLOG
+# define	NOKILLPG
+# define	USEINETATON
+# endif
+#endif
+
+#if	defined (__ANDROID__) || defined (__BIONIC__)
+#define	OSTYPE2			"ANDROID"
+#define	DEFKCODE		"utf8-iconv"
+#define	UTF8DOC
+#define	USETERMIO
+#define	NOFTPH
+#define	USEREADMTAB
+#define	USEPROCMNT
+#define	NOVSPRINTF
+#define	NOSYSLOG
+#define	NOKILLPG
+#define	NOGETPASS
+#define	NOGETPWENT
+#define	NOGETGRENT
 #endif
 
 #if	defined (__386BSD__) && !defined (OSTYPE)
@@ -639,6 +862,7 @@ typedef long	off_t;
 #define	USEMOUNTH
 #define	USEMNTINFO
 #define	USESETENV
+#define	ULONGIOCTL
 #define	USEMKTIME
 #define	USEWAITPID
 #define	USESIGPMASK
@@ -653,7 +877,7 @@ typedef long	off_t;
 # undef	CCCOMMAND
 # define	CCCOMMAND	"/bsd43/bin/cc"
 # endif
-#define	TERMCAPLIB	"-lcurses -ltermcap"
+#define	TERMCAPLIB		"-lcurses -ltermcap"
 #define	NOERRNO
 #define	NOFILEMODE
 #define	NOUNISTDH
@@ -666,6 +890,11 @@ typedef long	off_t;
 #define	SIGARGINT
 #endif
 
+#if	defined (__GNUC__) && defined (__clang__)
+#define	OSTYPE2			"LLVM"
+#define	NOUINT
+#endif
+
 /****************************************************************
  *	If your machine is not described above,	you should	*
  *	comment out below manually to apply your environment.	*
@@ -697,8 +926,9 @@ typedef long	off_t;
 /*	AUX		;A/UX (Apple) */
 /*	DGUX		;DG/UX AViiON (DG) */
 /*	UXPM		;UXP/M (Fujitsu) */
-/*	MIPS		;RISC/os (MIPS) */
+/*	UXPDS		;UXP/DS (Fujitsu) */
 /*	NEXTSTEP	;NEXTSTEP (NeXT) */
+/*	CYGWIN		;Cygwin */
 /*	LINUX		;Linux */
 /*	JCCBSD		;4.4BSD-Lite (JCC) */
 /*	FREEBSD		;FreeBSD */
@@ -707,16 +937,38 @@ typedef long	off_t;
 /*	BOW		;BSD on Windows (ASCII) */
 /*	OPENBSD		;OpenBSD */
 /*	DARWIN		;Darwin (Apple) */
+/*	MINIX		;MINIX */
+/*	ANDROID		;Android (aka Linux) */
 /*	ORG_386BSD	;386BSD */
+/*	MIPS		;RISC/os (MIPS) */
+/*	LLVM		;Low Level Virtual Machine */
 
 /* #define CODEEUC	;kanji code type is EUC */
+/* #define DEFKCODE	;default string for kanji code type */
+/* #define UTF8DOC	;kanji code type for documents is UTF-8 */
+/* #define UTF8LANG	;kanji code type if LANG environment value is UTF-8 */
+/* #define NOMULTIKANJI	;no need to support multiple kanji codes */
+/* #define PATHNOCASE	;pathname is case insensitive */
+/* #define COMMNOCASE	;shell command name is case insensitive */
+/* #define ENVNOCASE	;environment variable is case insensitive */
+/* #define NOUID	;have not uid, gid */
+/* #define NODIRLOOP	;any directory structure cannot loop */
+/* #define NOSYMLINK	;have not symbolic link */
+/* #define BSPATHDELIM	;path delimtor is backspace */
+/* #define USECRNL	;use CR-NL as end of line */
+/* #define CWDINPATH	;CWD is implicitly included in command path */
+/* #define DOUBLESLASH	;pathname starting with // has some special mean */
 /* #define USEMANLANG	;man(1) directory includes LANG environment value */
+/* #define LANGWIDTH	;column width of LANG used man(1) directory */
 /* #define SUPPORTSJIS	;cc(1) or man(1) supports Shift JIS perfectly */
 /* #define BSDINSTALL	;install(1) with option -c is valid like BSD */
 /* #define BSDINSTCMD	;command name except "install" to install like BSD */
-/* #define TARUSESPACE	;tar(1) uses space to devide file mode from UID */
+/* #define TARUSESPACE	;tar(1) uses space to divide file mode from UID */
 /* #define TARFROMPAX	;tar(1) comes from pax(1) */
+/* #define BUGGYMAKE	;make(1) assumes the same timestamp as earlier */
+/* #define BUGGYSIGNALH	;<signal.h> cannot be included for duplicated body */
 /* #define CPP7BIT	;cpp(1) cannot through 8bit */
+/* #define OLDARGINT	;cc(1) treats any old-fashioned argument as integer */
 /* #define CCCOMMAND	;fullpath of suitable cc(1) */
 /* #define EXTENDCCOPT	;additional option on cc(1) */
 /* #define CCOUTOPT	;option for output file name on cc(1) with -c */
@@ -724,13 +976,22 @@ typedef long	off_t;
 /* #define USETERMINFO	;use terminfo library instead of termcap */
 /* #define TERMCAPLIB	;library needed for termcap */
 /* #define REGEXPLIB	;library needed for regular expression */
+/* #define SOCKETLIB	;library needed for socket */
 /* #define EXTENDLIB	;library needed for the other extended */
 
 /* #define UNKNOWNFS	;use unsupported type FileSystem */
 
+/* #define NOFILEMACRO	;cc(1) cannot evaluate __FILE__ macro */
+/* #define NOFUNCMACRO	;cc(1) cannot evaluate __FUNCTION__ macro */
+/* #define NOLINEMACRO	;cc(1) cannot evaluate __LINE__ macro */
 /* #define FORCEDSTDC	;not defined __STDC__, but expect standard C */
 /* #define STRICTSTDC	;cannot allow K&R type function declaration */
+/* #define NOSTDC	;defined __STDC__, but expect traditional C */
+/* #define NOCONST	;defined __STDC__, but cannot use 'const' qualifier */
 /* #define NOVOID	;cannot use type 'void' */
+/* #define NOUINT	;cannot use type 'u_int' */
+/* #define NOLONGLONG	;cannot use type 'long long' */
+/* #define HAVELONGLONG	;have type 'long long' */
 /* #define NOUID_T	;uid_t, gid_t is not defined in <sys/types.h> */
 /* #define USEPID_T	;pid_t is defined in <sys/types.h> as process ID */
 /* #define DECLSIGLIST	;'sys_siglist[]' declared in <signal.h> */
@@ -742,22 +1003,31 @@ typedef long	off_t;
 /* #define NOUNISTDH	;have not <unistd.h> */
 /* #define NOSTDLIBH	;have not <stdlib.h> */
 /* #define NOTZFILEH	;have not <tzfile.h> */
-/* #define USELEAPCNT	;struct tzhead have tzh_leapcnt as leap second */
+/* #define USELEAPCNT	;'struct tzhead' has 'tzh_leapcnt' as leap second */
 /* #define USESELECTH	;use <sys/select.h> for select() */
 /* #define USESYSDIRH	;use <sys/dir.h> for DEV_BSIZE */
 /* #define USETIMEH	;use <time.h> for 'struct tm' */
 /* #define USESTDARGH	;use <stdarg.h> for va_list */
 /* #define USEMKDEVH	;use <sys/mkdev.h> for major()/minor() */
 /* #define USEMKNODH	;use <sys/mknod.h> for major()/minor() */
+/* #define USELOCKFH	;use <sys/lockf.h> for lockf() */
 /* #define USESGTTY	;use sgtty interface */
 /* #define USETERMIO	;use termio interface */
 /* #define USETERMIOS	;use termios interface */
+/* #define HAVECLINE	;'struct termios' has 'c_line' */
+/* #define USEDEVPTMX	;use /dev/ptmx and /dev/pts/? for pseudo terminal */
+/* #define USEDEVPTY	;use /dev/pty for pseudo terminal */
 /* #define USEDIRECT	;use 'struct direct' instead of dirent */
 /* #define SYSVDIRENT	;dirent interface behaves as System V */
-/* #define NODNAMLEN	;struct dirent haven't d_namlen */
-/* #define DNAMESIZE	;size of d_name in struct dirent */
-/* #define HAVETIMEZONE	;have extern valiable 'timezone' */
-/* #define NOTMGMTOFF	;struct tm haven't tm_gmtoff */
+/* #define NODNAMLEN	;'struct dirent' hasn't 'd_namlen' */
+/* #define NODRECLEN	;'struct dirent' hasn't 'd_reclen' but 'd_fd' */
+/* #define DNAMESIZE	;size of 'd_name' in 'struct dirent' */
+/* #define HAVETIMEZONE	;have extern variable 'timezone' */
+/* #define NOTMGMTOFF	;'struct tm' hasn't 'tm_gmtoff' */
+/* #define NOSTBLKSIZE	;'struct stat' hasn't 'st_blksize' */
+/* #define NOFTPH	;have not <arpa/ftp.h> */
+/* #define USEINSYSTMH	;use <netinet/in_systm.h> for <netinet/ip.h> */
+/* #define NOHADDRLIST	;'struct hostent' hasn't 'h_addr_list' */
 
 /* following 5 items are exclusive */
 /* #define USESTATVFSH	;use <sys/statvfs.h> as header of the FS status */
@@ -766,19 +1036,27 @@ typedef long	off_t;
 /* #define USEMOUNTH	;use <sys/mount.h> as header of the FS status */
 /* #define USEFSDATA	;use 'struct fs_data' as structure of hte FS status */
 
-/* #define USESTATVFS_T	;use 'statvfs_t' instead of 'struct statvfs' */
 /* #define USEFFSIZE	;'struct statfs' has 'f_fsize' as block size */
+/* #define NOFBLOCKS	;'struct statfs' hasn't 'f_blocks' */
+/* #define NOFBFREE	;'struct statfs' hasn't 'f_bfree' */
+/* #define NOFFILES	;'struct statfs' hasn't 'f_files' */
+/* #define USESTATVFS_T	;use 'statvfs_t' instead of 'struct statvfs' */
 /* #define STATFSARGS	;the number of arguments in statfs() */
+/* #define USEFSTATFS	;the fstatfs() instead of statfs() */
 
-/* following 8 items are exclusive */
+/* following 10 items are exclusive */
 /* #define USEMNTENTH	;use <mntent.h> as header of the mount entry */
 /* #define USEMNTTABH	;use <sys/mnttab.h> as header of the mount entry */
 /* #define USEGETFSSTAT	;use getfsstat() to get the mount entry */
+/* #define USEGETVFSTAT	;use getvfsstat() to get the mount entry */
 /* #define USEMNTCTL	;use mntctl() to get the mount entry */
 /* #define USEMNTINFOR	;use getmntinfo_r() to get the mount entry */
 /* #define USEMNTINFO	;use getmntinfo() to get the mount entry */
 /* #define USEGETMNT	;use getmnt() to get the mount entry */
 /* #define USEGETFSENT	;use getfsent() to get the mount entry */
+/* #define USEREADMTAB	;read directly /etc/mtab to get the mount entry */
+
+/* #define USEPROCMNT	;use /proc/mounts as MOUNTED */
 
 /* following 2 items are exclusive */
 /* #define USEVFCNAME	;'struct vfsconf' has 'vfc_name' as the mount type */
@@ -792,15 +1070,24 @@ typedef long	off_t;
 /* #define USERAND48	;use rand48() family instead of random() */
 /* #define USESETENV	;use setenv() instead of putenv() */
 /* #define NOSELECT	;have not select() */
+/* #define DEFFDSETSIZE	;pre-define FD_SETSIZE as OPEN_MAX for select() */
+/* #define SELECTRWONLY	;select() cannot respond with the file opened as r/w */
 /* #define NOVSPRINTF	;have not vsprintf() */
 /* #define NOTERMVAR	;have not termcap variables such as PC, ospeed, etc. */
 /* #define USEUTIME	;use utime() instead of utimes() */
 /* #define USEGETWD	;use getwd() instead of getcwd() */
+/* #define ULONGIOCTL	;the 2nd argument of ioctl() is unsigned long */
+/* #define NOFTRUNCATE	;have not ftruncate() */
+/* #define USEFCNTLOCK	;use fcntl() lock instead of flock() */
+/* #define USELOCKF	;use lockf() instead of flock() */
+/* #define NOFLOCK	;have neither flock() nor lockf() */
+/* #define NOSYSLOG	;have not syslog() */
 /* #define USETIMELOCAL	;have timelocal() as inverse of localtime() */
 /* #define USEMKTIME	;use mktime() instead of timelocal() */
 /* #define USESYSCONF	;use sysconf() for getting system configuration */
 /* #define USELLSEEK	;use _llseek() for 64bits width instead of lseek() */
 /* #define USEUNAME	;use uname() instead of gethostname() */
+/* #define NOKILLPG	;have not killpg() */
 /* #define USEWAITPID	;use waitpid() instead of wait3() */
 /* #define USESIGACTION	;use sigaction() instead of signal() */
 /* #define USESIGPMASK	;use sigprocmask() instead of sigsetmask() */
@@ -810,29 +1097,55 @@ typedef long	off_t;
 /* #define USETIMES	;use times() for getting process time */
 /* #define GETPGRPVOID	;getpgrp() needs void argument */
 /* #define USESETPGID	;use setpgid() instead of setpgrp() */
+/* #define USESETPGID	;use setpgid() instead of setpgrp() */
+/* #define NOSETPGRP	;have not setpgrp() */
+/* #define USETCGETPGRP	;use tcgetpgrp()/tcsetpgrp() instead of ioctl() */
 /* #define USESETVBUF	;use setvbuf() instead of setbuf() or setlinebuf() */
 /* #define SIGARGINT	;the 2nd argument function of signal() returns int */
 /* #define SIGFNCINT	;the 2nd argument function of signal() needs int */
 /* #define USESTRERROR	;use strerror() instead of sys_errlist[] */
 /* #define GETTODARGS	;the number of arguments in gettimeofday() */
+/* #define GETTODNULL	;the 2nd argument of gettimeofday() must be NULL */
+/* #define USESETSID	;use setsid() to set session ID */
+/* #define USEMMAP	;use mmap() to map files into memory */
+/* #define NOGETPASS	;have not getpass() */
+/* #define USESOCKLEN	;use socklen_t for bind()/connect()/accept() */
+/* #define NOSENDFLAGS	;send() cannot support flags */
+/* #define USEINETATON	;use inet_aton() instead of inet_addr() */
+/* #define USEINETPTON	;use inet_pton() instead of inet_addr() */
+/* #define NOGETPWENT	;have not getpwent() */
+/* #define NOGETGRENT	;have not getgrent() */
+/* #define USESETREUID	;use setreuid() to set effective user ID */
+/* #define USESETRESUID	;use setresuid() to set effective user ID */
+/* #define USESETREGID	;use setregid() to set effective group ID */
+/* #define USESETRESGID	;use setresgid() to set effective group ID */
+/* #define USEGETGROUPS	;use getgroups() to get group access list */
 
 /* #define SENSEPERSEC	;ratio of key sense per 1 second */
 /* #define WAITKEYPAD	;interval to wait after getting input of ESC [ms] */
-/* #define WAITMETA	;interval to wait after getting input of META [ms] */
-
-#include "config.h"
+/* #define WAITTERMINAL	;interval to wait for terminal response [ms] */
+/* #define WAITKANJI	;interval to wait after getting input of Kanji [ms] */
 
 
 /*                             */
 /* DO NOT DELETE or EDIT BELOW */
 /*                             */
 
+#ifdef	BSPATHDELIM
+#define	_SC_			'\\'
+#define	_SS_			"\\"
+#else
+#define	_SC_			'/'
+#define	_SS_			"/"
+#endif
+
 #if	defined (BSD43) || defined (BSD44)
 #define	BSD4
 #endif
 
 #if	defined (BSD44) || defined (SVR4)
 #define	POSIX
+#define	DEFFDSETSIZE
 #endif
 
 #ifdef	POSIX
@@ -849,10 +1162,13 @@ typedef long	off_t;
 #define	USESETPGID
 #define	USESETVBUF
 #define	USESTRERROR
+#define	USESETSID
+#define	USEMMAP
 #endif
 
 #ifdef	SVR4
 #define	SYSV
+#define	USESOCKLEN
 #endif
 
 #ifdef	SYSV
@@ -864,42 +1180,126 @@ typedef long	off_t;
 #define	HAVETIMEZONE
 #define	USERAND48
 #define	USEUNAME
+#define	NOKILLPG
 # if	!defined (USERE_COMP) && !defined (USEREGCOMP)
 # define	USEREGCMP
 # endif
 #endif
 
-#ifdef	USETERMIOS
-# ifdef	USETERMIO
-# undef	USETERMIO
+#if	defined (BSD4) && !defined (USEDEVPTY)
+#define	USEDEVPTY
+#endif
+
+#ifndef	USEDEVPTY
+#define	USEDEVPTMX
+#endif
+
+#if	defined (POSIX) || defined (SYSV)
+#define	USEFCNTLOCK
+#endif
+
+#if	defined (BSD4)
+#define	USEINSYSTMH
+#endif
+
+#if	defined (BSD4) && !defined (BSD43) && !defined (BSD44)
+#define	NOHADDRLIST
+#endif
+
+#if	defined (POSIX) || defined (BSD4)
+#define	USEGETGROUPS
+#endif
+
+#if	(!defined (__GNUC__) || defined (__clang__)) \
+&& !defined (__attribute__)
+#define	__attribute__(x)
+#endif
+
+#ifdef	__HOST_LANG__
+#undef	CODEEUC
+# if	!MSDOS \
+&& !defined (NEWS_OS6) && !defined (NEWS_OS4) && !defined (NEWS_OS3) \
+&& !defined (HPUX) && !defined (UNIOSB) \
+&& !defined (AIX) && !defined (AUX) \
+&& !defined (CYGWIN) && !defined (BOW) && !defined (DARWIN) \
+&& !defined (ANDROID) && !defined (ORG_386BSD)
+# define	CODEEUC
 # endif
 #endif
 
+
+/*                                 */
+/* Eval configurations by Configur */
+/*                                 */
+
+#include "config.h"
+
+#ifdef	USETERMIOS
+#undef	USETERMIO
+#endif
 #if	defined (USETERMIOS) || defined (USETERMIO)
-# ifdef	USESGTTY
-# undef	USESGTTY
-# endif
-#else
+#undef	USESGTTY
+#endif
+#if	!defined (USETERMIOS) && !defined (USETERMIO) && !defined (USESGTTY)
 #define	USESGTTY
 #endif
 
-#if	defined (__STDC__) || defined (FORCEDSTDC)
+#if	defined (__STDC__) && !defined (NOSTDC) && !defined (FORCEDSTDC)
+#define	FORCEDSTDC
+#endif
+
+#if	defined (__STRICT_ANSI__) && !defined (STRICTSTDC)
+#define	STRICTSTDC
+#endif
+
+#if	defined (FORCEDSTDC) && !defined (USESTDARGH)
 #define	USESTDARGH
 #endif
 
-#if	(defined (__STDC__) || defined (FORCEDSTDC)) \
-&& !defined (__STRICT_ANSI__) && !defined (STRICTSTDC)
-#define	__P_(args)	args
-#define	CONST		const
-#define	ALLOC_T		size_t
-#define	VOID_A		void
+#if	defined (__STDC__) && !defined (NOSTDC) \
+&& !defined (NOLONGLONG) && !defined (HAVELONGLONG)
+#define	HAVELONGLONG
+#endif
+
+#if	defined (FORCEDSTDC) && !defined (STRICTSTDC)
+#define	__P_(args)		args
+#else
+#define	__P_(args)		()
+#endif
+
+#if	defined (FORCEDSTDC) && !defined (NOCONST)
+#define	CONST			const
 #else
-#define	__P_(args)	()
 #define	CONST
-#define	ALLOC_T		unsigned
+#endif
+
+#ifdef	FORCEDSTDC
+#define	ALLOC_T			size_t
+#define	VOID_A			void
+#else
+#define	ALLOC_T			u_int
 #define	VOID_A
 #endif
 
+#ifdef	NOVOID
+#define	VOID
+#define	VOID_T			int
+#define	VOID_P			char *
+#define	VOID_C
+#else
+#define	VOID			void
+#define	VOID_T			void
+#define	VOID_P			void *
+#define	VOID_C			(void)
+#endif
+
+#ifdef	NOUINT
+typedef unsigned int		u_int;
+typedef unsigned char		u_char;
+typedef unsigned short		u_short;
+typedef unsigned long		u_long;
+#endif
+
 #if	defined (USEREGCMP) && !defined (REGEXPLIB)
 #define	REGEXPLIB		"-lPW"
 #endif
@@ -919,14 +1319,17 @@ typedef long	off_t;
 #endif
 #ifndef	TERMCAPLIB
 # ifdef	USETERMINFO
-# define	TERMCAPLIB		"-lcurses"
+# define	TERMCAPLIB	"-lcurses"
 # else
-# define	TERMCAPLIB		"-ltermlib"
+# define	TERMCAPLIB	"-ltermlib"
 # endif
 #endif
 #ifndef	REGEXPLIB
 #define	REGEXPLIB		""
 #endif
+#ifndef	SOCKETLIB
+#define	SOCKETLIB		""
+#endif
 #ifndef	EXTENDLIB
 #define	EXTENDLIB		""
 #endif
@@ -935,29 +1338,24 @@ typedef long	off_t;
 #endif
 
 #if	defined (USESTATVFSH)
-# ifdef	USESTATFSH
-# undef	USESTATFSH
-# endif
+#undef	USESTATFSH
 #endif
 
 #if	defined (USESTATVFSH) || defined (USESTATFSH)
-# ifdef	USEMOUNTH
-# undef	USEMOUNTH
-# endif
+#undef	USEMOUNTH
 #endif
 
 #if	defined (USESTATVFSH) || defined (USESTATFSH) || defined (USEMOUNTH)
-# ifdef	USEFSDATA
-# undef	USEFSDATA
-# endif
+#undef	USEFSDATA
 #endif
 
-#if	defined (USESTATVFSH) || defined (USESTATFSH) || defined (USEMOUNTH) \
-|| defined (USEFSDATA)
-# ifdef	USEVFSH
-# undef	USEVFSH
-# endif
-#else
+#if	defined (USESTATVFSH) || defined (USESTATFSH) \
+|| defined (USEMOUNTH) || defined (USEFSDATA)
+#undef	USEVFSH
+#endif
+
+#if	!defined (USESTATVFSH) && !defined (USESTATFSH) \
+&& !defined (USEMOUNTH) && !defined (USEFSDATA) && !defined (USEVFSH)
 #define	USEVFSH
 #endif
 
@@ -969,61 +1367,86 @@ typedef long	off_t;
 
 
 #if	defined (USEMNTTABH)
-# ifdef	USEGETFSSTAT
-# undef	USEGETFSSTAT
-# endif
+#undef	USEGETFSSTAT
 #endif
 
 #if	defined (USEMNTTABH) || defined (USEGETFSSTAT)
-# ifdef	USEMNTCTL
-# undef	USEMNTCTL
-# endif
+#undef	USEGETVFSTAT
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL)
-# ifdef	USEMNTINFOR
-# undef	USEMNTINFOR
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT)
+#undef	USEMNTCTL
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR)
-# ifdef	USEMNTINFO
-# undef	USEMNTINFO
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL)
+#undef	USEMNTINFOR
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO)
-# ifdef	USEGETMNT
-# undef	USEGETMNT
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR)
+#undef	USEMNTINFO
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO) || defined (USEGETMNT)
-# ifdef	USEGETFSENT
-# undef	USEGETFSENT
-# endif
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO)
+#undef	USEGETMNT
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO) || defined (USEGETMNT) \
-|| defined (USEGETFSENT)
-# ifdef	USEMNTENTH
-# undef	USEMNTENTH
-# endif
-#else
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT)
+#undef	USEGETFSENT
+#endif
+
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT) || defined (USEGETFSENT)
+#undef	USEREADMTAB
+#endif
+
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT) || defined (USEGETFSENT) \
+|| defined (USEREADMTAB)
+#undef	USEMNTENTH
+#endif
+
+#if	!defined (USEMNTTABH) && !defined (USEGETFSSTAT) \
+&& !defined (USEGETVFSTAT) && !defined (USEMNTCTL) && !defined (USEMNTINFOR) \
+&& !defined (USEMNTINFO) && !defined (USEGETMNT) && !defined (USEGETFSENT) \
+&& !defined (USEREADMTAB) && !defined (USEMNTENTH)
 #define	USEMNTENTH
 #endif
 
+#if	MSDOS
+#undef	USEVFSH
+#undef	USEMNTENTH
+#endif
+
+#if	defined (USEVFCNAME)
+#undef	USEFFSTYPE
+#endif
+
+
+#if	defined (USERE_COMP)
+#undef	USEREGCOMP
+#endif
+
+#if	defined (USERE_COMP) || defined (USEREGCOMP)
+#undef	USEREGCMP
+#endif
+
+
 #if	!defined (USEGETRUSAGE) && !defined (USETIMES) \
 && defined (USERESOURCEH)
 #define	USEGETRUSAGE
 #endif
 
 #ifndef	GETTODARGS
-#define	GETTODARGS	2
+#define	GETTODARGS		2
 #endif
 
-#endif	/* __MACHINE_H_ */
+#endif	/* !__MACHINE_H_ */
