@@ -1,47 +1,52 @@
 /*
  *	mkfuncno.c
  *
- *	Preprocesser for "funcno.h"
+ *	preprocesser for "funcno.h"
  */
 
-#include <stdio.h>
+#define	__FD_PRIMAL__
 #include "fd.h"
-
-#define	_TBL_(func, id, hlp, flg)	{NULL, id, NULL, 0}
-
+#include "types.h"
+#ifdef	_NOCATALOG
+#define	_TBL_(fn, id, hl, fl)	{NULL, id, NULL, 0}
+#else
+#define	_TBL_(fn, id, hl, fl)	{NULL, id, 0, 0}
+#endif
 #include "functabl.h"
 
+int main __P_((int, char *CONST []));
+
 
 /*ARGSUSED*/
 int main(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	FILE *fp;
 	int i, len;
 
 	if (!strcmp(argv[1], "-")) fp = stdout;
 	else if (!(fp = fopen(argv[1], "w"))) {
-		fprintf(stderr, "Cannot open file.\n");
+		fprintf(stderr, "%s: Cannot open.\n", argv[1]);
 		return(1);
 	}
 
 	fprintf(fp, "/*\n");
 	fprintf(fp, " *\t%s\n", (fp != stdout) ? argv[1] : "STDOUT");
 	fprintf(fp, " *\n");
-	fprintf(fp, " *\tFunction No. Table\n");
+	fprintf(fp, " *\tfunction No. table\n");
 	fprintf(fp, " */\n");
 	fprintf(fp, "\n");
 
-	for (i = 0; i < sizeof(funclist) / sizeof(functable); i++) {
+	for (i = 0; i < (int)sizeof(funclist) / sizeof(functable); i++) {
 		fprintf(fp, "#define\t%s\t", funclist[i].ident);
 		len = strlen(funclist[i].ident);
 		while ((len += 8) < 16) fputc('\t', fp);
-		fprintf(fp, "%d\n", i);
+		fprintf(fp, "\t%d\n", i);
 	}
-	fprintf(fp, "\n#define\tFUNCLISTSIZ\t%d\n", i);
+	fprintf(fp, "\n#define\tFUNCLISTSIZ\t\t%d\n", i);
 
-	if (fp != stdout) fclose(fp);
+	if (fp != stdout) VOID_C fclose(fp);
 
 	return(0);
 }
