@@ -1,38 +1,30 @@
 /*
  *	tree.c
  *
- *	Tree Mode Module
+ *	tree mode module
  */
 
-#include <ctype.h>
 #include "fd.h"
-#include "term.h"
 #include "func.h"
 #include "kanji.h"
 
+#define	DIRFIELD		3
+#define	TREEFIELD		(((dircountlimit > 0) \
+				? (n_column * 3) / 5 : n_column) - 2)
+#define	FILEFIELD		((dircountlimit > 0) \
+				? n_column - ((n_column * 3) / 5) - 3 : 0)
+#define	bufpos(y)		(((y) - 1) * (TREEFIELD + 1) * KANAWID)
+#define	bufptr(y)		(&(tr_scr[bufpos(y)]))
+
 #ifndef	_NOTREE
 
-#if	MSDOS && !defined (_NODOSDRIVE)
-extern int preparedrv __P_((int));
-#endif
-
-
 extern char fullpath[];
+extern int autoupdate;
 extern int win_x;
 extern int win_y;
-#ifndef	_NODOSDRIVE
-extern int lastdrv;
-#endif
-extern int sizeinfo;
 
-#define	DIRFIELD	3
-#define	TREEFIELD	(((dircountlimit > 0) \
-			? (n_column * 3) / 5 : n_column) - 2)
-#define	FILEFIELD	((dircountlimit > 0) ? (n_column * 2) / 5 - 3 : 0)
-#define	bufptr(y)	(&(tr_scr[(y - LFILETOP - 1) * (TREEFIELD + 1)]))
-
-static int NEAR evaldir __P_((char *, int));
-static treelist *NEAR maketree __P_((char *, treelist *, treelist *,
+static int NEAR evaldir __P_((CONST char *, int));
+static treelist *NEAR maketree __P_((CONST char *, treelist *, treelist *,
 		int, int *));
 static int NEAR _showtree __P_((treelist *, int, int, int));
 static VOID NEAR showtree __P_((VOID_A));
@@ -44,6 +36,7 @@ static int NEAR expandall __P_((treelist *));
 static int NEAR treeup __P_((VOID_A));
 static int NEAR treedown __P_((VOID_A));
 static int NEAR freetree __P_((treelist *, int));
+static int NEAR bottomtree __P_((VOID_A));
 static VOID NEAR _tree_search __P_((VOID_A));
 static int NEAR _tree_input __P_((VOID_A));
 static char *NEAR _tree __P_((VOID_A));
@@ -62,88 +55,88 @@ static treelist *tr_cur = NULL;
 
 
 static int NEAR evaldir(dir, disp)
-char *dir;
+CONST char *dir;
 int disp;
 {
 	DIR *dirp;
 	struct dirent *dp;
 	struct stat st;
-	char path[MAXPATHLEN];
-	int i, x, y, w, len, limit;
-#if	MSDOS
-	char *cp;
-#endif
+	char *cp, path[MAXPATHLEN];
+	int i, x, y, w, min, limit;
 
 	if ((limit = dircountlimit) <= 0) return(1);
 
-	if (disp) for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
-		locate(TREEFIELD + 2, i);
-		putch2('|');
-		putterm(l_clear);
+	min = filetop(win);
+	if (disp) for (i = 1; i < FILEPERROW; i++) {
+		Xlocate(TREEFIELD + 2, min + i);
+		VOID_C XXputch('|');
+		Xputterm(L_CLEAR);
 	}
-	strcpy(path, dir);
-#if	MSDOS
-	cp = path;
-	if (_dospath(cp)) cp += 2;
-	if (strcmp(cp, _SS_)) strcatdelim(path);
-#else
-	if (strcmp(path, _SS_)) strcatdelim(path);
-#endif
-	len = strlen(path);
-	i = x = 0;
-	y = LFILETOP + 1;
-	w = FILEFIELD / 2;
+
 	if (!(dirp = Xopendir(dir))) return(0);
+
+	Xstrcpy(path, dir);
+	cp = strcatdelim(path);
+	i = x = 0;
+	y = 1;
+	w = FILEFIELD;
+	if (limit > FILEPERROW - 1) w /= 2;
+
 	while ((dp = Xreaddir(dirp))) {
 		if (isdotdir(dp -> d_name)) continue;
-		strcpy(&(path[len]), dp -> d_name);
-		if (limit-- <= 0
-		|| (stat2(path, &st) >= 0
-		&& (st.st_mode & S_IFMT) == S_IFDIR)) {
+		if (strcatpath(path, cp, dp -> d_name) < 0) continue;
+
+		if (limit-- <= 0 || (stat2(path, &st) >= 0 && s_isdir(&st))) {
 			if (!disp) {
 				i++;
 				break;
 			}
 		}
 		else if (disp) {
-			locate(x + TREEFIELD + 4, y);
-			kanjiputs2(dp -> d_name, w, 0);
+			Xlocate(x + TREEFIELD + 4, min + y);
+			VOID_C XXcprintf("%^.*k", w, dp -> d_name);
 			i++;
-			if (++y >= LFILEBOTTOM) {
-				y = LFILETOP + 1;
+			if (++y >= FILEPERROW) {
+				y = 1;
 				x += w + 1;
 				if (x >= w * 2 + 2) break;
 			}
 		}
 	}
+	VOID_C Xclosedir(dirp);
+
 	if (disp && !i) {
-		locate(x + TREEFIELD + 4, LFILETOP + 1);
-		kanjiputs2("[No Files]", w, 0);
+		Xlocate(x + TREEFIELD + 4, min + 1);
+		cputstr(w, "[No Files]");
 	}
-	Xclosedir(dirp);
+
 	return(i);
 }
 
 /*ARGSUSED*/
 static treelist *NEAR maketree(path, list, parent, level, maxp)
-char *path;
+CONST char *path;
 treelist *list, *parent;
 int level, *maxp;
 {
+#ifdef	DEP_DOSEMU
+	char tmp[MAXPATHLEN];
+#endif
+#ifndef	NODIRLOOP
+	treelist *lp;
+#endif
 	DIR *dirp;
 	struct dirent *dp;
 	struct stat st;
-	char *cp, *dir, *subdir, cwd[MAXPATHLEN];
-#if	!MSDOS && !defined (_NODOSDRIVE)
-	char tmp[MAXPATHLEN];
-#endif
+	CONST char *cp, *subdir;
+	char *dir, cwd[MAXPATHLEN];
 	int i, len;
 
 	if ((level + 1) * DIRFIELD + 2 > TREEFIELD) {
 		*maxp = -2;
 		return(NULL);
 	}
-	if (intrkey()) {
+	if (intrkey(K_ESC)) {
 		*maxp = -1;
 		return(NULL);
 	}
@@ -152,46 +145,40 @@ int level, *maxp;
 		return(NULL);
 	}
 
-#if	MSDOS
-	if (_dospath(path)) path += 2;
-#endif
-	if (*path == _SC_) {
-		dir = strdup2(_SS_);
-		subdir = path + 1;
-		if (!*subdir) subdir = NULL;
-	}
-#if	!MSDOS && !defined (_NODOSDRIVE)
-	else if (_dospath(path)) {
-		dir = strndup2(path, 3);
-		subdir = path + 3;
-		if (!*subdir) subdir = NULL;
-	}
-#endif
-	else {
-		len = (cp = strdelim(path, 0)) ? cp - path : strlen(path);
-		dir = strndup2(path, len);
-		subdir = (cp) ? cp + 1 : NULL;
-	}
+	len = getpathtop(path, NULL, NULL);
+	cp = &(path[len]);
+
+	if (*cp == _SC_) cp++;
+	else cp = strdelim(path, 0);
+	len = (cp) ? cp - path : strlen(path);
+
+	dir = Xstrndup(path, len);
+	subdir = &(path[len]);
+	if (*subdir == _SC_) subdir++;
+	else if (!*subdir) subdir = NULL;
 
 	if (!subdir) len = 0;
-	else len = (cp = strdelim(subdir, 0)) ? cp - subdir : strlen(subdir);
+	else if ((cp = strdelim(subdir, 0))) len = cp - subdir;
+	else len = strlen(subdir);
 
 	*maxp = 0;
 	i = _chdir2(dir);
-	free(dir);
-	if (i < 0 || !(dirp = Xopendir("."))) return(NULL);
+	Xfree(dir);
+	if (i < 0 || !(dirp = Xopendir(curpath))) return(NULL);
 
 	i = 0;
 	while ((dp = Xreaddir(dirp))) {
-		if (isdotdir(dp -> d_name)
-		|| Xstat(nodospath(tmp, dp -> d_name), &st) < 0
-		|| (st.st_mode & S_IFMT) != S_IFDIR) continue;
+		if (isdotdir(dp -> d_name)) continue;
+		else if (Xstat(nodospath(tmp, dp -> d_name), &st) < 0
+		|| !s_isdir(&st))
+			continue;
+
 		list = b_realloc(list, *maxp, treelist);
 		if (!subdir) {
-			list[*maxp].name = strdup2(dp -> d_name);
+			list[*maxp].name = Xstrdup(dp -> d_name);
 			list[*maxp].sub = NULL;
 			list[*maxp].max = 0;
-#if	!MSDOS
+#ifndef	NODIRLOOP
 			list[*maxp].dev = st.st_dev;
 			list[*maxp].ino = st.st_ino;
 			list[*maxp].parent = parent;
@@ -199,50 +186,66 @@ int level, *maxp;
 			(*maxp)++;
 		}
 		else if (!strnpathcmp(dp -> d_name, subdir, len)
-		&& strlen(dp -> d_name) == len) {
-			list[0].name = strdup2(dp -> d_name);
+		&& !(dp -> d_name[len])) {
+			list[0].name = Xstrdup(dp -> d_name);
 			list[0].sub = &(list[0]);
 			list[0].max = 0;
-#if	!MSDOS
+#ifndef	NODIRLOOP
 			list[0].dev = st.st_dev;
 			list[0].ino = st.st_ino;
 			list[0].parent = parent;
 #endif
 			if (++(*maxp) >= 2) break;
 		}
-		else if (!i) {
-			i++;
-			list[1].name = NULL;
-			list[1].sub = NULL;
-			list[1].max = -1;
-#if	!MSDOS
-			list[1].dev = 0;
-			list[1].ino = 0;
-			list[1].parent = NULL;
+		else {
+			if (!*maxp) {
+				list[0].name = NULL;
+				list[0].sub = NULL;
+				list[0].max = -1;
+#ifndef	NODIRLOOP
+				list[0].dev = 0;
+				list[0].ino = 0;
+				list[0].parent = NULL;
 #endif
-			if (++(*maxp) >= 2) break;
+			}
+			if (!i) {
+				i++;
+				list[1].name = NULL;
+				list[1].sub = NULL;
+				list[1].max = -1;
+#ifndef	NODIRLOOP
+				list[1].dev = 0;
+				list[1].ino = 0;
+				list[1].parent = NULL;
+#endif
+				if (++(*maxp) >= 2) break;
+			}
 		}
 	}
-	Xclosedir(dirp);
-	if (list) for (i = 0; i < *maxp; i++) {
-#if	!MSDOS
-		treelist *lp;
+	VOID_C Xclosedir(dirp);
 
+	if (list) for (i = 0; i < *maxp; i++) {
+#ifndef	NODIRLOOP
 		if (!(list[i].name)) lp = NULL;
+# ifdef	DEP_URLPATH
+		else if (list[i].dev == (dev_t)-1 || list[i].ino == (ino_t)-1)
+			lp = NULL;
+# endif
 		else for (lp = parent; lp; lp = lp -> parent)
 			if (lp -> dev == list[i].dev
-			&& lp -> ino == list[i].ino) break;
+			&& lp -> ino == list[i].ino)
+				break;
 		if (lp) {
 			list[i].sub = NULL;
 			list[i].max = 0;
 		}
 		else
-#endif
+#endif	/* !NODIRLOOP */
 		if (list[i].sub)
 			list[i].sub = maketree(subdir, NULL, &(list[i]),
 				level + 1, &(list[i].max));
 		else {
-			if (list[i].max >= 0
+			if (list[i].max >= 0 && list[i].name
 			&& evaldir(nodospath(tmp, list[i].name), 0))
 				list[i].max = -1;
 		}
@@ -251,17 +254,17 @@ int level, *maxp;
 
 #if	MSDOS
 	if (_dospath(path)) path += 2;
-	if (*path == _SC_) {
-#else	/* !MSDOS */
-	if (*path == _SC_
-# ifndef	_NODOSDRIVE
-	|| _dospath(path)
-# endif
-	) {
-#endif	/* !MSDOS */
+#endif
+#ifdef	DEP_DOSEMU
+	if (*path == _SC_ || _dospath(path))
+#else
+	if (*path == _SC_)
+#endif
+	{
 		if (_chdir2(fullpath) < 0) error(fullpath);
 	}
-	else if (strcmp(path, ".") && _chdir2(cwd) < 0) error("..");
+	else if (isdotdir(path) != 2 && _chdir2(cwd) < 0) error(parentpath);
+
 	return(list);
 }
 
@@ -276,11 +279,13 @@ int max, nest, y;
 	tmp = y;
 	for (i = 0; i < max; i++) {
 		if (nest > 0) {
-			for (j = tmp; j < y; j++)
-			if (j > LFILETOP && j < LFILEBOTTOM)
-				*(bufptr(j) + (nest - 1) * DIRFIELD) = '|';
+			for (j = tmp; j < y; j++) {
+				if (j <= 0 || j >= FILEPERROW) continue;
+				cp = bufptr(j);
+				cp[(nest - 1) * DIRFIELD] = '|';
+			}
 		}
-		if (y > LFILETOP && y < LFILEBOTTOM) {
+		if (y > 0 && y < FILEPERROW) {
 			cp = bufptr(y);
 			if (nest > 0)
 				memcpy(&(cp[(nest - 1) * DIRFIELD]), "+--", 3);
@@ -289,12 +294,14 @@ int max, nest, y;
 				memcpy(&(cp[nest * DIRFIELD]), "...", tmp);
 			}
 			else {
-				tmp = strlen(list[i].name);
-				if (tmp > w) tmp = w;
-				memcpy(&(cp[nest * DIRFIELD]),
-					list[i].name, tmp);
-				if (list[i].max < 0)
-					cp[nest * DIRFIELD + tmp] = '>';
+				tmp = Xsnprintf(&(cp[nest * DIRFIELD]),
+					w * KANAWID + 1,
+					"%^.*s", w, list[i].name);
+#ifdef	CODEEUC
+				tmp = strlen(&(cp[nest * DIRFIELD]));
+#endif
+				cp[nest * DIRFIELD + tmp] =
+					(list[i].max < 0) ? '>' : ' ';
 			}
 		}
 		y++;
@@ -302,48 +309,47 @@ int max, nest, y;
 		if (w >= DIRFIELD && list[i].sub)
 			y = _showtree(list[i].sub, list[i].max, nest + 1, y);
 	}
+
 	return(y);
 }
 
 static VOID NEAR showtree(VOID_A)
 {
-	int i;
+	int i, min;
 
-	for (i = LFILETOP; i <= LFILEBOTTOM; i++) {
-		locate(0, i);
-		putterm(l_clear);
+	min = filetop(win);
+	for (i = 0; i < FILEPERROW; i++) {
+		Xlocate(0, min + i);
+		Xputterm(L_CLEAR);
 	}
-	for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
+	for (i = 1; i < FILEPERROW; i++) {
 		memset(bufptr(i), ' ', TREEFIELD);
 		bufptr(i)[TREEFIELD] = '\0';
 	}
-	_showtree(tr_root -> sub, 1, 0, tr_top);
-	for (i = LFILETOP + 1; i < LFILEBOTTOM; i++) {
-		locate(1, i);
-		if (i == tr_line) putterm(t_standout);
-		kanjiputs2(bufptr(i), TREEFIELD, 0);
-		if (i == tr_line) putterm(end_standout);
+	_showtree(tr_root -> sub, 1, 0, tr_top - min);
+	for (i = 1; i < FILEPERROW; i++) {
+		Xlocate(1, min + i);
+		attrputstr(TREEFIELD, bufptr(i), min + i == tr_line);
 	}
-	evaldir(treepath, 1);
+	VOID_C evaldir(treepath, 1);
 	keyflush();
 }
 
 static VOID NEAR treebar(VOID_A)
 {
-	locate(1, LFILETOP);
-	cputs2("Tree=");
-	kanjiputs2(treepath, n_column - 6, 0);
-	locate(0, L_MESLINE);
-	putterm(l_clear);
+	Xlocate(1, filetop(win));
+	VOID_C XXcprintf("Tree=%^-*.*k", n_column - 6, n_column - 6, treepath);
+	Xlocate(0, L_MESLINE);
+	Xputterm(L_CLEAR);
 }
 
 VOID rewritetree(VOID_A)
 {
-	tr_scr = realloc2(tr_scr, (FILEPERROW - 1) * (TREEFIELD + 1));
+	tr_scr = Xrealloc(tr_scr, bufpos(FILEPERROW));
 	searchtree();
 	showtree();
 	treebar();
-	tflush();
+	Xtflush();
 }
 
 static treelist *NEAR _searchtree(list, max, nest)
@@ -351,7 +357,7 @@ treelist *list;
 int max, nest;
 {
 	treelist *lp, *tmplp;
-	int i, j, w, len;
+	int i, w, len;
 
 	lp = NULL;
 	w = TREEFIELD - (nest * DIRFIELD) - 1;
@@ -359,38 +365,23 @@ int max, nest;
 		if (tr_bottom == tr_line) {
 			lp = tr_root;
 			tr_no = i;
-			if (list[i].name) strcpy(treepath, list[i].name);
+			if (list[i].name) Xstrcpy(treepath, list[i].name);
 			else *treepath = '\0';
 		}
 		tr_bottom++;
-		if (w >= DIRFIELD && list[i].sub) {
-			tmplp = _searchtree(list[i].sub, list[i].max,
-				nest + 1);
-			if (tmplp) {
-				lp = (tmplp == tr_root) ? &(list[i]) : tmplp;
-				len = strlen(list[i].name);
-#if	MSDOS
-				if (!_dospath(list[i].name)
-				|| list[i].name[2] != _SC_) len++;
-				for (j = strlen(treepath); j >= 0; j--)
-					treepath[j + len] = treepath[j];
-				memcpy(treepath, list[i].name, len - 1);
-#else	/* !MSDOS */
-				if (*list[i].name != _SC_
-# ifndef	_NODOSDRIVE
-				&& !_dospath(list[i].name)
-# endif
-				) len++;
-				for (j = strlen(treepath); j >= 0; j--)
-					treepath[j + len] = treepath[j];
-				if (*list[i].name != _SC_)
-					memcpy(treepath, list[i].name,
-						len - 1);
-#endif	/* !MSDOS */
-				treepath[len - 1] = _SC_;
-			}
-		}
+
+		if (w < DIRFIELD || !(list[i].sub)) continue;
+		tmplp = _searchtree(list[i].sub, list[i].max, nest + 1);
+		if (!tmplp) continue;
+
+		lp = (tmplp == tr_root) ? &(list[i]) : tmplp;
+		len = strlen(list[i].name);
+		if (len > 0 && list[i].name[len - 1] == _SC_) len--;
+		memmove(&(treepath[len + 1]), treepath, strlen(treepath) + 1);
+		memcpy(treepath, list[i].name, len);
+		treepath[len] = _SC_;
 	}
+
 	return(lp);
 }
 
@@ -409,11 +400,13 @@ treelist *list;
 
 	lp = NULL;
 	if (list -> sub) {
-		lp = (treelist *)malloc2(sizeof(treelist));
+		lp = (treelist *)Xmalloc(sizeof(treelist));
 		memcpy((char *)lp, (char *)&(list -> sub[0]),
 			sizeof(treelist));
-		for (i = 1; i < list -> max; i++)
-			if ((list -> sub[i]).name) free((list -> sub[i]).name);
+		for (i = 1; i < list -> max; i++) {
+			Xfree(list -> sub[i].name);
+			list -> sub[i].name = NULL;
+		}
 	}
 
 	if (_chdir2(treepath) < 0) {
@@ -421,33 +414,38 @@ treelist *list;
 		list -> max = 0;
 		return(1);
 	}
-	for (cp = treepath, i = 0; (cp = strdelim(cp, 0)); cp++, i++);
-	lptmp = maketree(".", list -> sub, list, i, &(list -> max));
+	for (cp = treepath, i = 0; (cp = strdelim(cp, 0)); cp++, i++)
+		/*EMPTY*/;
+	lptmp = maketree(curpath, list -> sub, list, i, &(list -> max));
 	if (_chdir2(fullpath) < 0) lostcwd(fullpath);
 	if (list -> max < 0) {
 		i = (list -> max < -1) ? 1 : 0;
 		if (!(list -> sub)) list -> max = -1;
 		else {
 			list -> max = 2;
-			(list -> sub[1]).name = NULL;
-			free(lp);
+			list -> sub[1].name = NULL;
+			Xfree(lp);
 		}
 		return(i);
 	}
 	if (list -> sub) {
-		for (i = 0; i < list -> max; i++)
-			if (!strpathcmp(lp -> name, lptmp[i].name)) break;
+		if (!(lp -> name)) i = list -> max;
+		else for (i = 0; i < list -> max; i++)
+			if (lptmp[i].name
+			&& !strpathcmp(lp -> name, lptmp[i].name))
+				break;
 		if (i < list -> max) {
-			free(lptmp[i].name);
+			Xfree(lptmp[i].name);
 			for (; i > 0; i--) memcpy((char *)&(lptmp[i]),
 					(char *)&(lptmp[i - 1]),
 					sizeof(treelist));
 			memcpy((char *)&(lptmp[0]), (char *)lp,
 				sizeof(treelist));
 		}
-		free(lp);
+		Xfree(lp);
 	}
 	list -> sub = lptmp;
+
 	return(1);
 }
 
@@ -460,46 +458,64 @@ treelist *list;
 	if (!list || list -> max >= 0) return(1);
 	if (!expandtree(list)) return(0);
 	if (!(list -> sub)) return(1);
-	cp = treepath + strlen(treepath);
-	if (cp > treepath + 1) {
+	cp = &(treepath[strlen(treepath)]);
+	if (cp > &(treepath[1])) {
 		strcatdelim(treepath);
 		cp++;
 	}
 	for (i = 0; i < list -> max; i++) {
-		strcpy(cp, (list -> sub[i]).name);
+		Xstrcpy(cp, list -> sub[i].name);
 		if (!expandall(&(list -> sub[i]))) return(0);
 	}
+
 	return(1);
 }
 
 static int NEAR treeup(VOID_A)
 {
-	if (tr_line > LFILETOP + 1) tr_line--;
-	else if (tr_top <= LFILETOP) tr_top++;
+	int min;
+
+	min = filetop(win);
+	if (tr_line > min + 1) tr_line--;
+	else if (tr_top <= min) tr_top++;
 	else return(-1);
 	searchtree();
+
 	return(0);
 }
 
 static int NEAR treedown(VOID_A)
 {
-	char *cp;
-	int oy, otop;
+	char *cp, *dir;
+	int min, oy, otop;
 
+	min = filetop(win);
 	oy = tr_line;
 	otop = tr_top;
-	if (tr_line < tr_bottom - 1 && tr_line < LFILEBOTTOM - 2)
+	if (tr_line < tr_bottom - 1 && tr_line < min + FILEPERROW - 2)
 		tr_line++;
-	else if (tr_bottom >= LFILEBOTTOM) tr_top--;
+	else if (tr_bottom >= min + FILEPERROW) tr_top--;
 	else return(-1);
 	searchtree();
 
-	if (!tr_cur || !(tr_cur -> sub)
-	|| (tr_cur -> sub[tr_no]).max >= 0
-	|| ((tr_cur -> sub[tr_no]).name)) return(0);
+	if (!tr_cur || !(tr_cur -> sub)) return(0);
+	if (tr_cur -> sub[tr_no].max >= 0 || tr_cur -> sub[tr_no].name)
+		return(0);
 
 	waitmes();
-	if ((cp = strrdelim(treepath, 0)) == strdelim(treepath, 0)) cp++;
+	dir = treepath;
+#ifdef	DEP_DOSPATH
+	if (_dospath(treepath)) dir += 2;
+#endif
+
+	cp = strrdelim(dir, 0);
+	if (cp == dir) cp++;
+#ifdef	DOUBLESLASH
+	else if (cp - treepath < isdslash(treepath)) cp++;
+#endif
+#ifdef	DEP_URLPATH
+	else if (cp - treepath < _urlpath(treepath, NULL, NULL)) cp++;
+#endif
 	*cp = '\0';
 	if (!expandtree(tr_cur)) {
 		tr_line = oy;
@@ -509,6 +525,7 @@ static int NEAR treedown(VOID_A)
 	}
 	searchtree();
 	redraw = 1;
+
 	return(0);
 }
 
@@ -520,13 +537,32 @@ int max;
 
 	for (i = n = 0; i < max; i++) {
 		n++;
-		if (list[i].name) free(list[i].name);
+		Xfree(list[i].name);
 		if (list[i].sub) n += freetree(list[i].sub, list[i].max);
 	}
-	free(list);
+	Xfree(list);
+
 	return(n);
 }
 
+static int NEAR bottomtree(VOID_A)
+{
+	int min, oy, otop;
+
+	min = filetop(win);
+	oy = tr_line;
+	otop = tr_top;
+	while (tr_line < min + FILEPERROW - 2) if (treedown() < 0) break;
+	if (tr_line >= tr_bottom - 1 && tr_top >= min + 1) {
+		tr_line = oy;
+		tr_top = otop;
+		searchtree();
+		return(-1);
+	}
+
+	return(0);
+}
+
 static VOID NEAR _tree_search(VOID_A)
 {
 	int oy, otop;
@@ -549,14 +585,15 @@ static int NEAR _tree_input(VOID_A)
 {
 	treelist *old;
 	char *cwd;
-	int ch, tmp, half;
+	int ch, min, tmp, half;
 
 	keyflush();
-	Xgetkey(-1, 0);
-	ch = Xgetkey(1, 0);
-	Xgetkey(-1, 0);
+	Xgetkey(-1, 0, 0);
+	ch = Xgetkey(1, 0, autoupdate);
+	Xgetkey(-1, 0, 0);
 
 	old = tr_cur;
+	min = filetop(win);
 	switch (ch) {
 		case K_UP:
 			treeup();
@@ -565,30 +602,26 @@ static int NEAR _tree_input(VOID_A)
 			treedown();
 			break;
 		case K_PPAGE:
+			if (bottomtree() < 0) break;
 			half = (FILEPERROW - 1) / 2;
-			tmp = LFILETOP + half + 1 - tr_line;
-			if (tr_top + half > LFILETOP + 1)
-				half = LFILETOP - tr_top + 1;
+			tmp = min + half + 1;
+			if (tr_top + half > min + 1) half = min - tr_top + 1;
 			if (half > 0) tr_top += half;
-			for (;tmp > 0; tmp--) if (treedown() < 0) break;
-			tr_line += tmp;
+			tr_line = tmp;
 			searchtree();
 			break;
 		case K_NPAGE:
+			if (bottomtree() < 0) break;
 			half = (FILEPERROW - 1) / 2;
-			tmp = half + (LFILETOP + half + 1) - tr_line;
-			while (tmp-- > 0) if (treedown() < 0) break;
-			tmp = tr_line - (LFILETOP + half + 1);
-			tr_line = LFILETOP + half + 1;
-			if (tr_bottom - tmp < LFILEBOTTOM - 1)
-				tmp = tr_bottom - LFILEBOTTOM + 1;
-			tr_top -= tmp;
+			tmp = min + half + 1;
+			while (half-- > 0) if (treedown() < 0) break;
+			tr_line = tmp;
 			searchtree();
 			break;
 		case K_BEG:
 		case K_HOME:
 		case '<':
-			tr_line = tr_top = LFILETOP + 1;
+			tr_line = tr_top = min + 1;
 			searchtree();
 			break;
 		case K_EOL:
@@ -614,7 +647,8 @@ static int NEAR _tree_input(VOID_A)
 			break;
 		case '\t':
 			if (!tr_cur || !(tr_cur -> sub)
-			|| (tr_cur -> sub[tr_no]).max >= 0) break;
+			|| tr_cur -> sub[tr_no].max >= 0)
+				break;
 			waitmes();
 			expandall(&(tr_cur -> sub[tr_no]));
 			searchtree();
@@ -622,8 +656,8 @@ static int NEAR _tree_input(VOID_A)
 			break;
 		case K_RIGHT:
 			if (!tr_cur || !(tr_cur -> sub)) break;
-			if ((tr_cur -> sub[tr_no]).max >= 0) {
-				if (!((tr_cur -> sub[tr_no]).sub)) break;
+			if (tr_cur -> sub[tr_no].max >= 0) {
+				if (!(tr_cur -> sub[tr_no].sub)) break;
 				if (treedown() < 0) break;
 			}
 			waitmes();
@@ -632,13 +666,13 @@ static int NEAR _tree_input(VOID_A)
 			redraw = 1;
 			break;
 		case K_LEFT:
-			if (tr_cur && (tr_cur -> sub)
-			&& (tr_cur -> sub[tr_no]).sub) {
-				tmp = freetree((tr_cur -> sub[tr_no]).sub,
-					(tr_cur -> sub[tr_no]).max);
+			if (tr_cur && tr_cur -> sub
+			&& tr_cur -> sub[tr_no].sub) {
+				tmp = freetree(tr_cur -> sub[tr_no].sub,
+					tr_cur -> sub[tr_no].max);
 				tr_bottom -= tmp;
-				(tr_cur -> sub[tr_no]).max = -1;
-				(tr_cur -> sub[tr_no]).sub = NULL;
+				tr_cur -> sub[tr_no].max = -1;
+				tr_cur -> sub[tr_no].sub = NULL;
 				redraw = 1;
 				break;
 			}
@@ -646,33 +680,40 @@ static int NEAR _tree_input(VOID_A)
 		case K_BS:
 			do {
 				if (treeup() < 0) break;
+				if (!tr_cur || !(tr_cur -> sub)) break;
 			} while (&(tr_cur -> sub[tr_no]) != old);
 			break;
 		case 'l':
-			if (!(cwd = inputstr(LOGD_K, 0, -1, NULL, 1))
-			|| !*(cwd = evalpath(cwd, 1))) break;
+			if (!(cwd = inputstr(LOGD_K, 0, -1, NULL, HST_PATH)))
+				break;
+			if (!*(cwd = evalpath(cwd, 0))) break;
 			if (chdir2(cwd) >= 0) {
-				free(cwd);
+				Xfree(cwd);
 				break;
 			}
 			warning(-1, cwd);
 			ch = '\0';
-			free(cwd);
+			Xfree(cwd);
 			break;
 		case K_CTRL('L'):
+#if	FD >= 3
+		case K_TIMEOUT:
+#endif
 			rewritefile(1);
 			break;
 		case K_ESC:
 			break;
 		default:
-			if (ch < 'A' || ch > 'Z') break;
+			if (ch >= K_MIN) break;
+			if (!Xisupper(ch)) break;
 			if (tr_line == tr_bottom - 1) {
-				tr_line = tr_top = LFILETOP + 1;
+				tr_line = tr_top = min + 1;
 				searchtree();
 			}
 			else do {
 				if (treedown() < 0) break;
-				tmp = toupper2(*((tr_cur -> sub[tr_no]).name));
+				if (!tr_cur || !(tr_cur -> sub)) break;
+				tmp = Xtoupper(*(tr_cur -> sub[tr_no].name));
 			} while (ch != tmp);
 			break;
 	}
@@ -682,42 +723,50 @@ static int NEAR _tree_input(VOID_A)
 
 static char *NEAR _tree(VOID_A)
 {
-#if	!MSDOS
-	struct stat st;
-# ifndef	_NODOSDRIVE
+#ifndef	NODIRLOOP
+# ifdef	DEP_DOSDRIVE
 	char tmp[MAXPATHLEN];
 # endif
-#endif
+	struct stat st;
+#endif	/* !NODIRLOOP */
 	char *cp, *cwd, path[MAXPATHLEN];
-	int ch, oy, otop;
+	int ch, min, len, oy, otop;
 
 	keyflush();
 	waitmes();
-	tr_root = (treelist *)malloc2(sizeof(treelist));
+	tr_root = (treelist *)Xmalloc(sizeof(treelist));
 	tr_root -> name = NULL;
 	tr_root -> max = 1;
-	tr_root -> sub = tr_cur = (treelist *)malloc2(sizeof(treelist));
-#if	!MSDOS
+	tr_root -> sub = tr_cur = (treelist *)Xmalloc(sizeof(treelist));
+#ifndef	NODIRLOOP
 	tr_root -> ino = 0;
 	tr_root -> parent = NULL;
 #endif
 	treepath = path;
-#if	MSDOS
-	tr_cur[0].name = malloc2(3 + 1);
-	strcpy(path, fullpath);
-	strncpy2(tr_cur[0].name, path, 3);
-#else	/* !MSDOS */
-# ifdef	_NODOSDRIVE
-	strcpy(path, fullpath);
-	tr_cur[0].name = strdup2(_SS_);
-# else
-	if (dospath("", path)) tr_cur[0].name = strndup2(path, 3);
-	else {
-		strcpy(path, fullpath);
-		tr_cur[0].name = strdup2(_SS_);
-	}
-# endif
 
+#ifdef	DEP_DOSEMU
+	if (dospath(nullstr, path)) len = 3;
+	else
+#endif
+	{
+#ifdef	DOUBLESLASH
+		if ((len = isdslash(fullpath))) /*EMPTY*/;
+		else
+#endif
+#ifdef	DEP_URLPATH
+		if ((len = _urlpath(fullpath, NULL, NULL))) /*EMPTY*/;
+		else
+#endif
+#if	MSDOS
+		len = 3;
+#else
+		len = 1;
+#endif
+		Xstrcpy(path, fullpath);
+	}
+	tr_cur[0].name = Xstrndup(path, len);
+
+#ifndef	NODIRLOOP
 	if (Xstat(nodospath(tmp, tr_cur[0].name), &st) < 0)
 		tr_cur[0].dev = tr_cur[0].ino = 0;
 	else {
@@ -725,30 +774,27 @@ static char *NEAR _tree(VOID_A)
 		tr_cur[0].ino = st.st_ino;
 	}
 	tr_cur[0].parent = NULL;
-#endif	/* !MSDOS */
+#endif
 	tr_cur[0].sub = maketree(path, NULL, &(tr_cur[0]),
 		0, &(tr_cur[0].max));
 
 	tr_line = 0;
-#if	MSDOS
-	cwd = path + 2;
-#else	/* !MSDOS */
-	cwd = path;
-# ifndef	_NODOSDRIVE
-	if (_dospath(cwd)) cwd += 2;
-# endif
-#endif	/* !MSDOS */
-	if (!strcmp(cwd, _SS_)) /*EMPTY*/;
+	if (path[len - 1] == _SC_) len--;
+	cwd = &(path[len]);
+
+	if (isrootpath(cwd)) /*EMPTY*/;
 	else for (cp = cwd; (cp = strdelim(cp, 0)); cp++, tr_line++)
 		if ((tr_line + 1) * DIRFIELD + 2 > TREEFIELD
-		|| !(tr_cur = &(tr_cur -> sub[0]))) break;
-	tr_line += (tr_top = LFILETOP + 1);
-	if (tr_line >= LFILEBOTTOM - 2) {
-		tr_top += LFILEBOTTOM - 2 - tr_line;
-		tr_line = LFILEBOTTOM - 2;
+		|| !(tr_cur = &(tr_cur -> sub[0])))
+			break;
+	min = filetop(win);
+	tr_line += (tr_top = min + 1);
+	if (tr_line >= min + FILEPERROW - 2) {
+		tr_top -= tr_line - (min + FILEPERROW - 2);
+		tr_line = min + FILEPERROW - 2;
 	}
 
-	tr_scr = malloc2((FILEPERROW - 1) * (TREEFIELD + 1));
+	tr_scr = Xmalloc(bufpos(FILEPERROW));
 	searchtree();
 	showtree();
 	win_x = 0;
@@ -758,63 +804,71 @@ static char *NEAR _tree(VOID_A)
 		otop = tr_top;
 		redraw = 0;
 		win_y = tr_line;
-		locate(win_x, win_y);
-		tflush();
+		Xlocate(win_x, win_y);
+		Xtflush();
 
 		if ((ch = _tree_input()) == 'l') {
-			free(tr_scr);
+			Xfree(tr_scr);
 			freetree(tr_root, 1);
 			return(fullpath);
 		}
 
 		if (redraw || tr_top != otop) showtree();
 		else if (oy != tr_line) {
-			locate(1, tr_line);
-			putterm(t_standout);
-			kanjiputs2(bufptr(tr_line), TREEFIELD, 0);
-			putterm(end_standout);
-			locate(1, oy);
-			if (stable_standout) putterm(end_standout);
-			else kanjiputs2(bufptr(oy), TREEFIELD, 0);
-			evaldir(path, 1);
+			Xlocate(1, tr_line);
+			attrputstr(TREEFIELD, bufptr(tr_line - min), 1);
+			Xlocate(1, oy);
+			if (stable_standout) Xputterm(END_STANDOUT);
+			else cputstr(TREEFIELD, bufptr(oy - min));
+			VOID_C evaldir(path, 1);
 		}
 	} while (ch != K_ESC && ch != K_CR);
 
-	free(tr_scr);
+	Xfree(tr_scr);
 	freetree(tr_root, 1);
 	treepath = NULL;
 	if (ch == K_ESC) return(NULL);
-	return(strdup2(path));
+
+	return(Xstrdup(path));
 }
 
 /*ARGSUSED*/
-char *tree(cleanup, ddp)
-int cleanup, *ddp;
+char *tree(cleanup, drvp)
+int cleanup, *drvp;
 {
+#ifdef	DEP_PSEUDOPATH
+	int drv;
+#endif
 	char *path, *dupfullpath;
 
 	if (FILEPERROW < WFILEMINTREE) {
 		warning(0, NOROW_K);
 		return(NULL);
 	}
-	dupfullpath = strdup2(fullpath);
+#ifdef	DEP_PSEUDOPATH
+	if ((drv = preparedrv(fullpath, NULL, NULL)) < 0) {
+		warning(-1, fullpath);
+		return(NULL);
+	}
+#endif
+	dupfullpath = Xstrdup(fullpath);
 	do {
 		path = _tree();
 	} while (path == fullpath);
-	if (ddp) {
-#ifndef	_NODOSDRIVE
-# if	MSDOS
-		if (lastdrv < 0) *ddp = preparedrv(dospath(path, NULL));
-		else
-# endif
-		*ddp = lastdrv;
-		lastdrv = -1;
-#endif
+#ifdef	DEP_PSEUDOPATH
+	if (path && drvp && (*drvp = preparedrv(path, NULL, NULL)) < 0) {
+		warning(-1, path);
+		Xfree(path);
+		path = NULL;
 	}
+#endif
 	if (chdir2(dupfullpath) < 0) lostcwd(NULL);
-	free(dupfullpath);
-
+	Xfree(dupfullpath);
+#ifdef	DEP_PSEUDOPATH
+	shutdrv(drv);
+#endif
 	if (cleanup) rewritefile(1);
+
 	return(path);
 }
 #endif	/* !_NOTREE */
